// constant class -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "Constant.h"
#include "LeafValue.h"
#include "Property.h"

/** @file Constant.C
 * Constant literals in expressions
 */

Constant::Constant (class Value& value) :
  Expression (), myValue (&value)
{
  Expression::setType (myValue->getType ());
}

Constant::~Constant ()
{
  delete myValue;
}

bool
Constant::operator== (const class Constant& other) const
{
  if (getValue ().getKind () != other.getValue ().getKind ())
    return false;
  return getValue () == other.getValue ();
}

bool
Constant::operator< (const class Constant& other) const
{
  if (getValue ().getKind () < other.getValue ().getKind ())
    return true;
  if (other.getValue ().getKind () < getValue ().getKind ())
    return false;
  return getValue () < other.getValue ();
}

class Value*
Constant::do_eval (const class Valuation&) const
{
  return myValue->copy ();
}

bool
Constant::depends (const class VariableSet&,
		   bool) const
{
  return false;
}

bool
Constant::forVariables (bool (*)
			(const class Expression&,void*),
			void*) const
{
  return true;
}

class Ltl*
Constant::toFormula (class Property& property)
{
  assert (myValue->getKind () == Value::vLeaf &&
	  myValue->getType ().getKind () == Type::tBool);
  return property.addConstant (static_cast<const class LeafValue&>(*myValue));
}

#ifdef EXPR_COMPILE
# include "CExpression.h"

void
Constant::compileCompatible (class CExpression& cexpr,
			     unsigned indent,
			     const class VariableSet&,
			     const char* value) const
{
  if (getType ()->getNumValues () != 1) {
    class StringBuffer& out = cexpr.getOut ();
    out.indent (indent), out.append ("if (");
    myValue->compileEqual (out, indent + 4, value, false, true, true);
    out.append (")\n");
    cexpr.compileError (indent + 2, errComp);
  }
}

void
Constant::compile (class CExpression& cexpr,
		   unsigned indent,
		   const char* lvalue,
		   const class VariableSet*) const
{
  myValue->compileInit (lvalue, indent, cexpr.getOut ());
}

#endif // EXPR_COMPILE

void
Constant::display (const class Printer& printer) const
{
  myValue->display (printer);
}
